#include "main.h"
#include <Deskbar.h>
#include <StorageKit.h>
#include <string.h>

#define SERIALIZER_MIME "application/x-vnd.Tools4b.MediaSerializer"
#define MEDIA_MIME "application/x-vnd.Be.MediaPlayer"


StopButton *StopButton::Instantiate(BMessage *archive) {
	if(validate_instantiation(archive, "StopButton"))
		return new StopButton(archive);
	else return NULL;
}

StopButton::StopButton(BMessage *archive)
	: BView(archive) {
}

StopButton::StopButton()
	: BView(BRect(0,0,14,14), "StopButton", B_FOLLOW_NONE, B_WILL_DRAW) {
	SetViewColor(184,184,184);
	SetLowColor(216,216,216);
	SetHighColor(64,64,64);
}

void StopButton::Draw(BRect urect) {
	BRect r= Bounds();
	FillEllipse(BRect(r.left, r.top+1, r.right, r.bottom-1), B_SOLID_LOW);
	StrokeEllipse(BRect(r.left, r.top+1, r.right, r.bottom-1));
	FillRect(BRect(5,5,9,9));
}

void StopButton::MouseDown(BPoint pos) {
	BMessenger(SERIALIZER_MIME).SendMessage(B_QUIT_REQUESTED);
	BMessenger(MEDIA_MIME).SendMessage(B_QUIT_REQUESTED);
}

status_t StopButton::Archive(BMessage *archive, bool deep) const {
	BView::Archive(archive, deep);
	archive->AddString("add_on", SERIALIZER_MIME);
	archive->AddString("class", "StopButton");
	return B_OK;
}

//--------------------------------------------------//

MediaSerializer::MediaSerializer()
	: BApplication(SERIALIZER_MIME)
	, launchingMedia(false) {
	be_roster->StartWatching(BMessenger(this), B_REQUEST_QUIT| B_REQUEST_ACTIVATED);

	StopButton stopb;
	BDeskbar().AddItem(&stopb);
}

void MediaSerializer::MessageReceived(BMessage *msg) {
	if(msg->what== B_SOME_APP_QUIT) {
		const char *mime= msg->FindString("be:signature");
		if(!strcmp(mime, MEDIA_MIME))
			PlayNext();
	}
	if(msg->what== B_SOME_APP_ACTIVATED) {
		const char *mime= msg->FindString("be:signature");
		if(launchingMedia && !strcmp(mime, MEDIA_MIME)) {
			launchingMedia= false;
			be_roster->ActivateApp(frontApp.team);
		}
	}
}

void MediaSerializer::ArgvReceived(int32 argc, char **argv) {
	BMessage refsm;
	for(int i=1; i<argc; i++) {
		entry_ref ref;
		BEntry entry(argv[i]);
		entry.GetRef(&ref);
		refsm.AddRef("refs", &ref);
	}
	RefsReceived(&refsm);
}

void MediaSerializer::RefsReceived(BMessage *msg) {
	for(int i= playlist.CountItems()-1; i>=0; i--)
		delete playlist.RemoveItem(i);

	entry_ref ref;
	for(int i=0; msg->FindRef("refs", i, &ref)== B_OK; i++) {
		BDirectory dir;
		if(dir.SetTo(&ref)== B_OK)
			AddDir(&dir);
		else playlist.AddItem(new entry_ref(ref));
	}
	PlayNext();
}

void MediaSerializer::AddDir(BDirectory *dir) {
	entry_ref ref;
	while(dir->GetNextRef(&ref)== B_OK) {
		BDirectory dir;
		if(dir.SetTo(&ref)== B_OK)
			AddDir(&dir);
		else playlist.AddItem(new entry_ref(ref));
	}
}

void MediaSerializer::PlayNext() {
	entry_ref *ref= (entry_ref*) playlist.ItemAt(0);
	if(ref) {
		BMessage msg(B_REFS_RECEIVED);
		msg.AddRef("refs", ref);

		be_roster->GetActiveAppInfo(&frontApp);
		be_roster->Launch(MEDIA_MIME, &msg);
		launchingMedia= true;

		delete playlist.RemoveItem(0.0);
	} else PostMessage(B_QUIT_REQUESTED);
}

bool MediaSerializer::QuitRequested() {
	BDeskbar().RemoveItem("StopButton");
	return true;
}

//--------------------------------------------------//

int main() {
	MediaSerializer().Run();
	return B_OK;
}
